#!/usr/bin/env bash

# (C) Copyright Collin J. Doering 2017
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# File: keyboard.sh
# Author: Collin J. Doering <collin.doering@rekahsoft.ca>
# Date: Feb 27, 2017

KEYWORD='AT Translated Set 2'
CFG_FILE="$HOME/.keyboard"

case "$1" in
    on)
        # Turn keyboard on
        if [ -f "$CFG_FILE" ]; then
            INP=($(cat $CFG_FILE))
        else
            echo "Error! No config file '${CFG_FILE}'"
            exit 1
        fi

        ID=${INP[0]}
        MASTER=${INP[1]}

        xinput reattach "$ID" "$MASTER"
        ;;
    off)
        # Turn keyboard off
        IFS=$'\n'
        IDS=($(xinput --list --id-only))
        NAMES=($(xinput --list --name-only))
        unset IFS

        ID=''
        for i in "${!IDS[@]}"; do
            if [[ "${NAMES[$i]}" =~ "$KEYWORD" ]]; then
                ID="${IDS[$i]}"
                break
            fi
        done

        MASTER=$(xinput --list | grep "$KEYWORD" | head -n1 | tail -c4 | head -c1)
        echo "$ID $MASTER" > "$CFG_FILE"

        xinput float "$ID"
        ;;
    *)
        echo "Error! Invalid use of keyboard.sh"
        exit 1
        ;;
esac

# Exit gracefully
exit 0
